from contextvars import ContextVar

from .models import User

current_user_context: ContextVar[User] = ContextVar('current_user_context', default=None)

# Inside your middleware
# req_id.set(user)

# Inside other functions, even different files, you import that variable
# req_id.get()



from fastapi import HTTPException
from jwt import DecodeError
from .init import get_caseoor


def get_casdoor_if_ready(throw_http_exception: bool = True):
    casdoor = get_caseoor()
    if not casdoor:
        if throw_http_exception:
            raise HTTPException(status_code=500, detail="Casdoor is not boot")
        else:
            raise Exception("Casdoor is not boot")
    return casdoor



def validate_user_by_token(token: str, throw_http_exception: bool = True):
    casdoor = get_casdoor_if_ready(throw_http_exception)
    try:
        return casdoor.parse_jwt_token(token)
    except Exception as e:
        if not throw_http_exception:
            raise e
        if isinstance(e, DecodeError):
            raise HTTPException(status_code=401, detail="Invalid token")
        raise e



from fastapi import APIRouter, Depends

from . import deps, models

casdoor_router = APIRouter()

@casdoor_router.get('/me', tags=['casdoor'], response_model=models.User)
def route_get_current_user(user: models.User = Depends(deps.get_current_user)):
    return user




from typing import Annotated

from fastapi import Depends, HTTPException
from fastapi.security import HTTPAuthorizationCredentials, HTTPBearer

from .context import current_user_context
from .validators import validate_user_by_token
from .models import User


security = HTTPBearer()


async def get_current_user_or_none_without_check(credentials: Annotated[HTTPAuthorizationCredentials, Depends(security)]):
    if credentials.scheme != "Bearer":
        return None
    token = credentials.credentials
    user_dict = validate_user_by_token(token)
    if not user_dict:
        return None
    user = User.from_token_dict(user_dict)
    current_user_context.set(user)
    return user


async def get_current_user_without_check(user: Annotated[User | None, Depends(get_current_user_or_none_without_check)]):
    if not user:
        raise HTTPException(status_code=401, detail="Invalid token")
    return user


async def get_current_user(user: Annotated[User, Depends(get_current_user_without_check)]):
    if user.isDeleted or user.isForbidden:
        raise HTTPException(status_code=403, detail="You are not allowed to login.")
    return user


async def need_admin_user(user: Annotated[User, Depends(get_current_user)]):
    if not user.isAdmin:
        raise HTTPException(status_code=403, detail="You are not the admin")
    return user



from datetime import datetime
from pydantic import BaseModel

from .utils import str2datetime


class User(BaseModel):
    """
    example
    {
		"owner": "org_woai_test",
		"name": "test_user",
		"createdTime": "2023-03-30T18:38:25+08:00",
		"updatedTime": "",
		"id": "5cf501cc-d7d0-4975-89d3-25b4d3b486c7",
		"type": "normal-user",
		"password": "",
		"passwordSalt": "",
		"displayName": "测试用户",
		"firstName": "",
		"lastName": "",
		"avatar": "https://cdn.casbin.org/img/casbin.svg",
		"permanentAvatar": "",
		"email": "i@iaside.com",
		"emailVerified": false,
		"phone": "13120165229",
		"location": "",
		"address": [],
		"affiliation": "Example Inc.",
		"title": "",
		"idCardType": "",
		"idCard": "",
		"homepage": "",
		"bio": "",
		"region": "",
		"language": "",
		"gender": "",
		"birthday": "",
		"education": "",
		"score": 0,
		"karma": 0,
		"ranking": 2,
		"isDefaultAvatar": false,
		"isOnline": false,
		"isAdmin": false,
		"isGlobalAdmin": false,
		"isForbidden": false,
		"isDeleted": false,
		"signupApplication": "application_woai_test",
		"hash": "",
		"preHash": "",
		"createdIp": "",
		"lastSigninTime": "",
		"lastSigninIp": "",
		"ldap": "",
		"properties": {},
		"roles": [],
		"permissions": [],
		"lastSigninWrongTime": "",
		"signinWrongTimes": 0,
		"tokenType": "access-token",
		"tag": "staff",
		"iss": "https://casdoor.test.sinkstars.com",
		"sub": "5cf501cc-d7d0-4975-89d3-25b4d3b486c7",
		"aud": [
			"97c018a6692f932ab6d1"
		],
		"exp": 1680261803,
		"nbf": 1680175403,
		"iat": 1680175403,
		"jti": "admin/64e60a04-49ca-4452-beac-c445f924bac5"
	}
    """
    id: str
    name: str
    displayName: str
    firstName: str
    lastName: str
    email: str
    emailVerified: bool
    phone: str

    avatar: str
    permanentAvatar: str
    isDefaultAvatar: bool

    owner: str
    signupApplication: str

    type: str
    roles: list[str]
    permissions: list[str]
    tag: str

    language: str | None

    isOnline: bool
    isAdmin: bool
    isForbidden: bool
    isDeleted: bool

    createdTime: datetime | None
    updatedTime: datetime | None

    def from_token_dict(token_dict: dict):
        token_dict['createdTime'] = str2datetime(token_dict['createdTime'])
        token_dict['updatedTime'] = str2datetime(token_dict['updatedTime'])
        # if token_dict.get('exp'):
        #     token_dict['exp'] = datetime.fromtimestamp(token_dict['exp'])
        return User(**token_dict)



from casdoor import AsyncCasdoorSDK

from .settings import AuthSettings, AuthSettingFromEnvFile


casdoor: AsyncCasdoorSDK = None
auth_settings: AuthSettings = None


def init_casdoor(settings: AuthSettings = None):
    if not settings:
        settings = AuthSettingFromEnvFile()

    global auth_settings
    auth_settings = settings

    global casdoor
    casdoor = AsyncCasdoorSDK(
        auth_settings.AUTH_ENDPOINT,
        auth_settings.AUTH_CLIENT_ID,
        auth_settings.AUTH_CLIENT_SECRET,
        auth_settings.AUTH_CERT,
        auth_settings.AUTH_ORG_NAME,
        auth_settings.AUTH_APP_NAME,
    )


def get_caseoor() -> AsyncCasdoorSDK:
    return casdoor


def get_auth_settings() -> AuthSettings:
    return auth_settings



from pydantic_settings import BaseSettings


class AuthSettings(BaseSettings):
    """Casdoor settings."""
    AUTH_ENDPOINT: str
    AUTH_CLIENT_ID: str
    AUTH_CLIENT_SECRET: str
    AUTH_CERT: str
    AUTH_ORG_NAME: str
    AUTH_APP_NAME: str
    # AUTH_FORCE_USER_ACTIVATED = False


class AuthSettingFromEnvFile(AuthSettings):
    class Config:
        env_file = '.env'

